/*****************************************************************************
 * Copyright (c) 2022, BoLian Technologies Inc.
 *
 * All rights reserved.
 * ****************************************************************************
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * BoLian' name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY NATIONS "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL NATIONS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ****************************************************************************/

/**
 * @file delay.c
 * @author BoLian
 * @version v1.0.0
 *
 * @copyright Copyright (c) 2022, BoLian Technologies Inc. All rights reserved.
 */

#include "delay.h"
#include "n32g45x_tim.h"
//#include "bms_def.h"
/*
Options->Linker->Misc Controls 
--keep=delay_us_soft
Not optimized by the compiler
*/
void delay_us_soft(uint32_t count){
  volatile uint32_t i,j;
  for(i=0;i<count;i++)
    for(j=0;j<0x2000;j++);
}


/**
 * @brief  us delay  program function.
 * @param nus: the setting us value.
 *   note: The system clock should be an integer multiple of 1M .
 *   It is ensure accuracy .
 */
void systick_delay_us(u32 nus)
{       
    u32 temp;       
    SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);    //select system clock   
    SysTick->LOAD=nus*(SystemCoreClock/1000000); //time relode           
    SysTick->VAL=0x00;        //clear timer value
    SysTick->CTRL|=SysTick_CTRL_ENABLE_Msk ;  //Start countdown  
    do
    {
        temp=SysTick->CTRL;
    }
    while(temp&0x01&&!(temp&(1<<16)));//wait for the time reach 
    SysTick->CTRL&=~SysTick_CTRL_ENABLE_Msk;       //close the count
    SysTick->VAL =0X00;       //clear timer value    
}

/**
 * @brief  us delay  program function.
 * @param nus: the setting us value.
 *   note: The system clock should be an integer multiple of 1M .
 *   It is ensure accuracy .
 *   nms(1~116) 
 */
void systick_delay_ms(u16 nms)
{                 
    u32 temp;
    SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);  //select system clock   
    SysTick->LOAD=(u32)nms*((SystemCoreClock/1000000)*1000);//time relode(SysTick->LOAD is 24bit)(16,777,216)
    SysTick->VAL =0x00;           //clear timer value
    SysTick->CTRL|=SysTick_CTRL_ENABLE_Msk ;          //Start countdown  
    do
    {
        temp=SysTick->CTRL;
    }
    while(temp&0x01&&!(temp&(1<<16)));//wait for the time reach    
    SysTick->CTRL&=~SysTick_CTRL_ENABLE_Msk;       //close the count
    SysTick->VAL =0X00;       //clear timer value               
} 

//뼶ʱ
void TIM7_Delay_ms(u16 xms)
{
    TIM_TimeBaseInitType TIM_TimeBaseStructure;
    NVIC_InitType NVIC_InitStructure;
    uint16_t PrescalerValue;
    RCC_EnableAPB1PeriphClk(RCC_APB1_PERIPH_TIM7, ENABLE);
    /* Enable the TIM6 global Interrupt */
    NVIC_InitStructure.NVIC_IRQChannel                   = TIM6_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority        = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd                = ENABLE;

    NVIC_Init(&NVIC_InitStructure);
    /* Compute the prescaler value */
    PrescalerValue = 7200-1;//HCLK=144MAPB1=4PCLK1=36MTIM234567*2=72MHzTIM7=10000HZ
    
	TIM_TimeBaseStructure.Period    = 199;
    TIM_TimeBaseStructure.Prescaler = 0;  
    TIM_TimeBaseStructure.ClkDiv    = 0;
    TIM_TimeBaseStructure.CntMode   = TIM_CNT_MODE_UP;
	TIM_InitTimeBase(TIM7,&TIM_TimeBaseStructure);
    
	TIM7->STS = 0;
    
    /* Prescaler configuration */
    TIM_ConfigPrescaler(TIM7, PrescalerValue, TIM_PSC_RELOAD_MODE_IMMEDIATE);
    /* TIM6 enable update irq */
    TIM_ConfigInt(TIM6, TIM_INT_UPDATE, ENABLE);
    
    /* TIM7 enable counter */
    TIM_Enable(TIM7, ENABLE); 
    
	while((TIM7->STS & 0x01)!=SET){;}
    
    TIM_Enable(TIM7, DISABLE);
	TIM7->CNT = 0;    
}


void delay_us(uint32_t count){ 
    systick_delay_us(count);
}

uint32_t dly100ms,dly10ms;
/* 144MHzƵ£SysTick->LOAD һ24bitļĴֵΪ16,777,216
 * ʱֵΪ116ms,ʱֵ100msֵ˴
*/
void delay_ms(uint32_t count){
    int n;
    if(count<=100)
    {
        systick_delay_ms(count);
    }else
    {
        dly100ms = count/100;
        dly10ms  = count%100;
        for(n=0;n<dly100ms;n++)
        {
            systick_delay_ms(100);
        }
        if(dly10ms){
          systick_delay_ms(dly10ms);
        }
    }
}
